print("import: 'ruptures'")
import ruptures

print("import: 'ruptures.costs'")
import ruptures.costs

print("import: 'ruptures.datasets'")
import ruptures.datasets

print("import: 'ruptures.detection'")
import ruptures.detection

print("import: 'ruptures.metrics'")
import ruptures.metrics

print("import: 'ruptures.show'")
import ruptures.show

print("import: 'ruptures.utils'")
import ruptures.utils

